package org.hidetake.elshard.demo.controller.demo.internal;

import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.hidetake.elshard.demo.model.Book;
import org.hidetake.elshard.demo.service.AuthenticationService;
import org.hidetake.elshard.demo.service.BookService;
import org.hidetake.elshard.task.Configuration;
import org.hidetake.elshard.task.Context;
import org.hidetake.elshard.task.s3.TaskChainController;
import org.hidetake.elshard.task.splitter.SizeSplitter;
import org.slim3.controller.validator.Validators;

public class AddController extends TaskChainController
{

	private final AuthenticationService authenticationService = new AuthenticationService();
	private final BookService bookService = new BookService();

	@Override
	protected void configure(Configuration configuration)
	{
		configuration.setSplitter(new SizeSplitter(50));
	}

	@Override
	public List<String> input(Context context) throws Exception
	{
		String userId = authenticationService.getUserId();
		if(!validate()) {
			response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
			response.setContentType("text/plain");
			response.setCharacterEncoding("UTF-8");
			response.getWriter().append(errors.toString());
			return null;
		}
		context.addParameter("user", userId);
		return Arrays.asList(asString("books").split("\n"));
	}
	
	@Override
	public List<String> run(List<String> input, Context context)
	{
		String user = asString("user");
		
		for(String line : input) {
			String[] parts = line.split("\t");
			if(parts.length < 1) {
				continue;
			}
			Book book = new Book();
			book.setTitle(parts[0]);
			if(parts.length > 1) {
				book.setAuthor(parts[1]);
			}
			book.setUser(user);
			bookService.add(book);
		}
		
		return null;
	}

	private boolean validate()
	{
		Validators v = new Validators(request);
		v.add("books", v.required());
		return v.validate();
	}

}
