package org.hidetake.elshard.demo.controller.test;

import java.io.PrintWriter;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.hidetake.elshard.demo.meta.BookMeta;
import org.slim3.controller.Controller;
import org.slim3.controller.Navigation;
import org.slim3.datastore.Datastore;

import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.Query.SortDirection;

public class ByScatterController extends Controller
{

	@Override
	protected Navigation run() throws Exception
	{
		BookMeta m = BookMeta.get();
		
		response.setContentType("text/plain");
		response.setCharacterEncoding("UTF-8");
		PrintWriter writer = response.getWriter();

		writer.println("# sort by __scatter__ asc");
		List<Key> scatterKeys = Datastore.query(m)
				.sort("__scatter__", SortDirection.ASCENDING)
				.asKeyList();
		for(Key key : scatterKeys) {
			writer.println(key.getId());
		}
		writer.println();
		
		Collections.sort(scatterKeys, new Comparator<Key>() {
			@Override
			public int compare(Key o1, Key o2)
			{
				return o1.compareTo(o2);
			}
		});
		
		Set<Key> expected = new HashSet<Key>(Datastore.query(m).asKeyList());
		Set<Key> actual = new HashSet<Key>(expected.size());
		
		int scatterCount = scatterKeys.size();
		if(scatterCount > 0) {
			{
				Key endKey = scatterKeys.get(0);
				List<Key> shardKeys = Datastore.query(m)
						.filter(m.key.lessThan(endKey))
						.asKeyList();

				actual.addAll(shardKeys);
				writer.print("- ");
				writer.print(endKey);
				writer.print(": ");
				writer.println(shardKeys.size());
			}
			
			for(int i=0; i < scatterCount-1; i++) {
				Key startKey = scatterKeys.get(i);
				Key endKey = scatterKeys.get(i+1);
				List<Key> shardKeys = Datastore.query(m)
						.filter(m.key.greaterThanOrEqual(startKey), m.key.lessThan(endKey))
						.asKeyList();
				
				actual.addAll(shardKeys);
				writer.print(startKey);
				writer.print(" - ");
				writer.print(endKey);
				writer.print(": ");
				writer.println(shardKeys.size());
			}
			
			{
				Key startKey = scatterKeys.get(scatterCount-1);
				List<Key> shardKeys = Datastore.query(m)
						.filter(m.key.greaterThanOrEqual(startKey))
						.asKeyList();
				
				actual.addAll(shardKeys);
				writer.print(startKey);
				writer.print(" - : ");
				writer.println(shardKeys.size());
			}
		}

		writer.print("expected.size(): ");
		writer.println(expected.size());
		
		writer.print("actual.size(): ");
		writer.println(actual.size());
		
		writer.print("expected.containsAll(actual): ");
		writer.println(expected.containsAll(actual));

		writer.print("actual.containsAll(expected): ");
		writer.println(actual.containsAll(expected));
		
		writer.println();
		writer.println("expected.removeAll(actual): ");
		expected.removeAll(actual);
		for(Key key : expected) {
			writer.println(key);
		}
		
		return null;
	}

}
