package org.hidetake.elshard.demo.controller.test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.hidetake.elshard.demo.model.ForkRecord;
import org.slim3.controller.Controller;
import org.slim3.controller.ControllerConstants;
import org.slim3.controller.Navigation;
import org.slim3.datastore.Datastore;
import org.slim3.memcache.Memcache;
import org.slim3.util.IntegerUtil;

import com.google.appengine.api.taskqueue.Queue;
import com.google.appengine.api.taskqueue.QueueFactory;
import com.google.appengine.api.taskqueue.TaskOptions;
import com.google.appengine.api.users.User;
import com.google.appengine.api.users.UserService;
import com.google.appengine.api.users.UserServiceFactory;

public class ForkTestController extends Controller
{
	private static final String PARAM_CONDITION_INDEX = "i";
	private static final String PARAM_TASK_COUNT = "c";
	private static final String PARAM_SLICE = "s";
	private static final String PARAM_WAIT = "w";
	private static final String PARAM_BASE_TIME = "b";
	private static final String PARAM_SPINUP = "u";
	private static final String PARAM_EMAIL = "m";
	private static final Logger logger = Logger.getLogger(ForkTestController.class.getName());

	private static final List<Condition> conditions = new ArrayList<Condition>();
	static {
		// try warm up
		List<Condition> unit = Arrays.asList(
				// 10000sec
				new Condition(5, 2000),
				new Condition(10, 1000),
				new Condition(20, 500),
				new Condition(50, 200),
				new Condition(100, 100),

				// 20000sec
				new Condition(10, 2000),
				new Condition(20, 1000),
				new Condition(25, 800),
				new Condition(50, 400),
				new Condition(100, 200),

				// 30000sec
				new Condition(10, 3000),
				new Condition(20, 1500),
				new Condition(30, 1000),
				new Condition(50, 600),
				new Condition(100, 300),

				// 60000sec
				new Condition(10, 6000),
				new Condition(20, 3000),
				new Condition(30, 2000),
				new Condition(60, 1000),
				new Condition(100, 600)
			);
		conditions.addAll(unit);
		conditions.addAll(unit);
		conditions.addAll(unit);
		conditions.addAll(unit);
		conditions.addAll(unit);
	}

	private static class Condition
	{
		public final int taskCount;
		public final int wait;
		public Condition(int taskCount, int wait)
		{
			this.taskCount = taskCount;
			this.wait = wait;
		}
	}

	private final Queue queue = QueueFactory.getDefaultQueue();
	private long startTimeMillis;
	private String email;
	
	@Override
	protected Navigation run() throws Exception
	{
		this.startTimeMillis = System.currentTimeMillis();
		this.email = asString(PARAM_EMAIL);
		if(this.email == null) {
			UserService userService = UserServiceFactory.getUserService();
			User user = userService.getCurrentUser();
			if(user == null) {
				// oops
				logger.severe("no user data");
				return null;
			}
			this.email = user.getEmail();
		}

		if(param(PARAM_SLICE) == null) {
			int conditionIndex = IntegerUtil.toPrimitiveInt(param(PARAM_CONDITION_INDEX));
			if(conditionIndex < 0 || conditionIndex >= conditions.size()) {
				return finale();
			}
			return enqueue(conditionIndex);
		}
		else {
			return task();
		}
	}

	private Navigation enqueue(int conditionIndex) throws Exception
	{
		boolean spinup = false;
		if(applicationScope(PARAM_SPINUP) == null) {
			applicationScope(PARAM_SPINUP, true);
			spinup = true;
		}
		
		Condition condition = conditions.get(conditionIndex);
		for(int i=0; i < condition.taskCount; i++) {
			TaskOptions task = TaskOptions.Builder
					.withMethod(TaskOptions.Method.GET)
					.url("/test/forkTest")
					.param(PARAM_CONDITION_INDEX, Integer.toString(conditionIndex))
					.param(PARAM_EMAIL, email)
					.param(PARAM_TASK_COUNT, Integer.toString(condition.taskCount))
					.param(PARAM_SLICE, Integer.toString(i+1))
					.param(PARAM_WAIT, Integer.toString(condition.wait))
					.param(PARAM_SPINUP, Boolean.toString(spinup))
					.param(PARAM_BASE_TIME, Long.toString(this.startTimeMillis));
			this.queue.add(task);
		}
		
		long timeElapsed = System.currentTimeMillis() - this.startTimeMillis;
		
		logger.info("running on "+applicationScope(ControllerConstants.UUID_KEY));
		logger.info(condition.taskCount+" tasks forked in "+timeElapsed+" ms");
		return null;
	}

	private Navigation task() throws Exception
	{
		int conditionIndex = asInteger(PARAM_CONDITION_INDEX);
		int taskCount = asInteger(PARAM_TASK_COUNT);
		int waitMillis = asInteger(PARAM_WAIT);
		long baseTime = asLong(PARAM_BASE_TIME);
		int slice = asInteger(PARAM_SLICE);
		
		Thread.sleep(waitMillis);
		
		long endTimeFromQueued = System.currentTimeMillis() - baseTime;
		
		logger.info("running on "+applicationScope(ControllerConstants.UUID_KEY));
		logger.info("this task is #"+slice+" of "+taskCount);
		logger.info("this task running from "+(this.startTimeMillis - baseTime)+
				" (ms) to "+endTimeFromQueued+" (ms)");
		
		long count = Memcache.increment(baseTime, 1, 0);
		if(count == taskCount) {
			// terminated
			ForkRecord record = new ForkRecord();
			record.setConcurrency(taskCount);
			record.setElapsedTime(endTimeFromQueued);
			record.setStartTime(new Date(baseTime));
			record.setWait(waitMillis);
			record.setSpinup(asBoolean(PARAM_SPINUP));
			Datastore.put(record);
			
			// start next condition
			TaskOptions task = TaskOptions.Builder
					.withMethod(TaskOptions.Method.GET)
					.url("/test/forkTest")
					.param(PARAM_CONDITION_INDEX, Integer.toString(conditionIndex+1))
					.param(PARAM_EMAIL, param(PARAM_EMAIL))
					.countdownMillis(3000);
			this.queue.add(task);
		}
		
		return null;
	}

	private Navigation finale() throws Exception
	{
		Properties props = new Properties();
		Session session = Session.getDefaultInstance(props, null);
		Message msg = new MimeMessage(session);
        msg.setFrom(new InternetAddress(this.email));
        msg.addRecipient(Message.RecipientType.TO, new InternetAddress(this.email));
        msg.setSubject(ForkTestController.class.getName());
        msg.setText("Job finished.");
        Transport.send(msg);
		
		return null;
	}
}
