package org.hidetake.elshard.demo.controller.test;

import java.util.logging.Logger;

import org.slim3.controller.Controller;
import org.slim3.controller.Navigation;
import org.slim3.controller.validator.Validators;

import com.google.appengine.api.taskqueue.Queue;
import com.google.appengine.api.taskqueue.QueueFactory;
import com.google.appengine.api.taskqueue.TaskOptions;

/**
 * タスクのパラメータサイズ上限をチェックしてみる．
 * @see http://localhost:8888/test/taskParamSizeTest?size=10102 is OK
 * @see http://elshard-demo.appspot.com/test/taskParamSizeTest?size=10102 is OK
 * @author hidetake.org
 */
public class TaskParamSizeTestController extends Controller
{
	
	private static final Logger logger = Logger.getLogger(TaskParamSizeTestController.class.getName());

	@Override
	protected Navigation run() throws Exception
	{
		if(request.getHeader("X-AppEngine-TaskName") == null) {
			return first();
		}
		else {
			return task();
		}
	}

	private Navigation first() throws Exception
	{
		if(!validate()) {
			response.setContentType("text/plain");
			response.setCharacterEncoding("UTF-8");
			response.getWriter().append(errors.toString());
			response.setStatus(400);
			return null;
		}
		
		int size = asInteger("size");
		StringBuilder builder = new StringBuilder();
		for(int i=0; i < size; i++) {
			builder.append(i % 10);
		}
		
		Queue queue = QueueFactory.getDefaultQueue();
		TaskOptions task = TaskOptions.Builder
				.withMethod(TaskOptions.Method.POST)
				.url("/test/taskParamSizeTest")
				.param("d", builder.toString());
		queue.add(task);
		return null;
	}

	private Navigation task() throws Exception
	{
		String data = asString("d");
		logger.info("length=" + data.length());
		return null;
	}

	private boolean validate()
	{
		Validators v = new Validators(request);
		v.add("size", v.required(), v.integerType());
		return v.validate();
	}

}
