package org.hidetake.elshard.demo.controller.tweet.fetch;

import java.io.PrintWriter;
import java.util.List;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletResponse;

import org.hidetake.elshard.demo.model.Tweet;
import org.hidetake.elshard.demo.service.TweetService;
import org.slim3.controller.Controller;
import org.slim3.controller.Navigation;
import org.slim3.controller.validator.Validators;

import twitter4j.Paging;
import twitter4j.TwitterException;

/**
 * 一度のリクエストでツイートを取得する。
 * 
 * @author hidetake.org
 */
public class OnceController extends Controller
{

	private final Logger logger = Logger.getLogger(OnceController.class.getName());
	private final TweetService tweetService = new TweetService();

	@Override
	public Navigation run() throws Exception
	{
		if(!validate()) {
			throw new IllegalArgumentException(errors.toString());
		}
		String userName = asString("userName");
		
		for(int page = 1;;) {
			try {
				Paging paging = new Paging(page, 100);
				List<Tweet> tweets = tweetService.fetchAndSaveTimeline(userName, paging);
				if(tweets == null) {
					break;
				}
				logger.info("Processed "+tweets.size()+" tweets in page "+page);
				page++;
			}
			catch(TwitterException e) {
				logger.warning(e.getLocalizedMessage());
				if(e.exceededRateLimitation()) {
					int waitSecs = e.getRateLimitStatus().getResetTimeInSeconds();
					logger.warning("Waiting "+waitSecs+" seconds for rate limit reset");
					Thread.sleep(waitSecs*1000L);
				}
			}
		}
		
		return null;
	}

	@Override
	protected Navigation handleError(Throwable error) throws Throwable
	{
		response.setContentType("text/plain");
		response.setCharacterEncoding("UTF-8");
		response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
		PrintWriter writer = response.getWriter();
		writer.println(error.getLocalizedMessage());
		error.printStackTrace();
		return null;
	}

	private boolean validate()
	{
		Validators v = new Validators(request);
		v.add("userName", v.required());
		return v.validate();
	}

}
