package org.hidetake.elshard.demo.controller.tweet.parse;

import java.util.List;

import org.hidetake.elshard.demo.meta.TweetSurfaceMeta;
import org.hidetake.elshard.demo.model.TweetSurface;
import org.hidetake.elshard.demo.service.TweetSurfaceService;
import org.hidetake.jsonizer.Jsonizer;
import org.hidetake.jsonizer.PropertyGetter;
import org.slim3.controller.Controller;
import org.slim3.controller.Navigation;
import org.slim3.controller.validator.Validators;

public class ResultController extends Controller
{

	private final TweetSurfaceService tweetSurfaceService = new TweetSurfaceService();

	@Override
	public Navigation run() throws Exception
	{
		if(!validate()) {
			response.setContentType("text/plain");
			response.setCharacterEncoding("UTF-8");
			response.getWriter().append(errors.toString());
			response.setStatus(400);
			return null;
		}
		
		Jsonizer jsonizer = new Jsonizer();
		jsonizer.type(TweetSurface.class).with(new PropertyGetter<TweetSurface>()
		{
			private final TweetSurfaceMeta m = TweetSurfaceMeta.get();
			@Override
			public void get(TweetSurface e)
			{
				property(e.getSurface()).as(m.surface);
				property(e.getFeatures()).as(m.features);
				property(e.getCount()).as(m.count);
			}
		});
		
		List<TweetSurface> tweetSurfaces = tweetSurfaceService.get(param("userName"), 100);
		jsonizer.jsonize(tweetSurfaces).send(response);
		return null;
	}
	
	private boolean validate()
	{
		Validators v = new Validators(request);
		v.add("userName", v.required(), v.maxlength(50));
		return v.validate();
	}

}