package org.hidetake.elshard.demo.mapper.tweet;

import java.util.List;

import net.reduls.gomoku.Morpheme;
import net.reduls.gomoku.Tagger;

import org.apache.hadoop.io.NullWritable;
import org.hidetake.elshard.demo.meta.TweetMeta;
import org.hidetake.elshard.demo.model.Tweet;
import org.hidetake.elshard.demo.model.TweetSurface;
import org.hidetake.elshard.demo.util.TweetUtil;

import com.google.appengine.api.datastore.Entity;
import com.google.appengine.api.datastore.Key;
import com.google.appengine.tools.mapreduce.AppEngineMapper;

/**
 * ツイートの形態素をカウントする．
 *
 * @author hidetake.org
 */
public class ParseTweet extends AppEngineMapper<Key, Entity, NullWritable, NullWritable>
{

	public static final String COUNTER_GROUP_SURFACE = "surface";
	public static final String CONFIG_USER_NAME = "org.hidetake.elshard.demo.mapper.tweet.ParseTweet.UserName";
	
	@Override
	public void map(Key key, Entity entity, Context context)
	{
		TweetMeta m = TweetMeta.get();
		Tweet tweet = m.entityToModel(entity);
		
		String targetUserName = context.getConfiguration().get(CONFIG_USER_NAME);
		if(targetUserName != null && !tweet.getUserName().equals(targetUserName)) {
			return;
		}
		
		String status = TweetUtil.cleanup(tweet.getStatus());
		List<Morpheme> morphemes = Tagger.parse(status);
		
		for(Morpheme morpheme : morphemes) {
			// 表記と品詞をカウンタに入れる．
			String surfaceKey = TweetSurface.createKeyString(
					tweet.getUserName(),
					morpheme.surface,
					morpheme.feature);
			context.getCounter(COUNTER_GROUP_SURFACE, surfaceKey).increment(1);
		}
	}

}
