package org.hidetake.elshard.demo.model;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.mapreduce.Counters;
import org.slim3.datastore.Attribute;
import org.slim3.datastore.Model;

import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.Text;
import com.google.appengine.tools.mapreduce.ConfigurationXmlUtil;

/**
 * The model for appengine-mapreduce state.
 * 
 * @author hidetake.org
 */
@Model
public class MapReduceState
{

	public static enum Status
	{
		ACTIVE, DONE
	}

	@Attribute(primaryKey = true)
	private Key jobID;

	private long activeShardCount;
	private Text chart;
	private Text configuration;
	@Attribute(lob = true)
	private byte[] countersMap;
	private long lastPollTime;
	private String name;
	private double progress;
	private long shardCount;
	private long startTime;
	private Status status;

	public Key getJobID()
	{
		return jobID;
	}
	public void setJobID(Key jobID)
	{
		this.jobID = jobID;
	}
	public long getActiveShardCount()
	{
		return activeShardCount;
	}
	public void setActiveShardCount(long activeShardCount)
	{
		this.activeShardCount = activeShardCount;
	}
	public Text getChart()
	{
		return chart;
	}
	public void setChart(Text chart)
	{
		this.chart = chart;
	}
	public Text getConfiguration()
	{
		return configuration;
	}
	public Configuration getConfigurationInstance()
	{
		return ConfigurationXmlUtil.getConfigurationFromXml(getConfiguration().getValue());
	}
	public void setConfiguration(Text configuration)
	{
		this.configuration = configuration;
	}
	public byte[] getCountersMap()
	{
		return countersMap;
	}
	public Counters getCounters()
	{
	    try {
	    	// deserialize writable
			Counters counters = new Counters();
			counters.readFields(new DataInputStream(new ByteArrayInputStream(getCountersMap())));
			return counters;
		}
		catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	public void setCountersMap(byte[] countersMap)
	{
		this.countersMap = countersMap;
	}
	public long getLastPollTime()
	{
		return lastPollTime;
	}
	public void setLastPollTime(long lastPollTime)
	{
		this.lastPollTime = lastPollTime;
	}
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	public double getProgress()
	{
		return progress;
	}
	public void setProgress(double progress)
	{
		this.progress = progress;
	}
	public long getShardCount()
	{
		return shardCount;
	}
	public void setShardCount(long shardCount)
	{
		this.shardCount = shardCount;
	}
	public long getStartTime()
	{
		return startTime;
	}
	public void setStartTime(long startTime)
	{
		this.startTime = startTime;
	}
	public Status getStatus()
	{
		return status;
	}
	public void setStatus(Status status)
	{
		this.status = status;
	}

}
