package org.hidetake.elshard.demo.model;

import org.slim3.datastore.Attribute;
import org.slim3.datastore.Datastore;
import org.slim3.datastore.Model;

import com.google.appengine.api.datastore.Key;

/**
 * Tweet surface model.
 * 
 * @author hidetake.org
 */
@Model
public class TweetSurface
{

	@Attribute(primaryKey = true)
	private Key key;
	
	private String userName;
	private String surface;
	private String features;
	private long count;

	@SuppressWarnings("unused")
	private String userNameAndCount;
	@SuppressWarnings("unused")
	private String userNameAndFeaturesAndCount;

	public static String createKeyString(String userName, String surface, String feature)
	{
		StringBuilder builder = new StringBuilder();
		builder.append(userName);
		builder.append("\t\t");
		builder.append(surface);
		builder.append("\t\t");
		builder.append(feature);
		return builder.toString();
	}

	public static Key createKey(String userName, String surface, String feature)
	{
		return Datastore.createKey(TweetSurface.class,
				createKeyString(userName, surface, feature));
	}

	public static TweetSurface createFromKeyString(String keyString)
	{
		return createFromKey(Datastore.createKey(TweetSurface.class, keyString));
	}

	public static TweetSurface createFromKey(Key key)
	{
		String[] parts = key.getName().split("\t\t");
		TweetSurface result = new TweetSurface();
		result.setKey(key);
		result.setUserName(parts[0]);
		result.setSurface(parts[1]);
		result.setFeatures(parts[2]);
		return result;
	}

	public Key getKey()
	{
		return key;
	}

	public void setKey(Key key)
	{
		this.key = key;
	}

	public String getUserName()
	{
		return userName;
	}

	public void setUserName(String userName)
	{
		this.userName = userName;
	}

	public String getSurface()
	{
		return surface;
	}

	public void setSurface(String surface)
	{
		this.surface = surface;
	}

	public String getFeatures()
	{
		return features;
	}

	public void setFeatures(String feature)
	{
		this.features = feature;
	}

	public long getCount()
	{
		return count;
	}

	public void setCount(long count)
	{
		this.count = count;
	}

	@Deprecated
	public String getUserNameAndCount()
	{
		return String.format("%s\t%08d", this.userName, this.count);
	}

	@Deprecated
	public void setUserNameAndCount(String userNameAndCount)
	{
		this.userNameAndCount = userNameAndCount;
	}

	@Deprecated
	public String getUserNameAndFeaturesAndCount()
	{
		return String.format("%s\t%s\t%08d", this.userName, this.features, this.count);
	}

	@Deprecated
	public void setUserNameAndFeaturesAndCount(String userNameAndFeaturesAndCount)
	{
		this.userNameAndFeaturesAndCount = userNameAndFeaturesAndCount;
	}

}
