package org.hidetake.elshard.demo.service;

import org.slim3.util.AppEngineUtil;

import com.google.appengine.api.users.User;
import com.google.appengine.api.users.UserService;
import com.google.appengine.api.users.UserServiceFactory;

/**
 * Authentication service.
 * 
 * @author hidetake.org
 */
public class AuthenticationService
{

	private final UserService userService = UserServiceFactory.getUserService();

	/**
	 * ログイン中のユーザIDを返す．
	 * @return ユーザID
	 * @throws IllegalStateException ログインしていない場合
	 */
	public final String getUserId()
	{
		if(!AppEngineUtil.isServer()) {
			return "testId";
		}
		User user = userService.getCurrentUser();
		if(user == null) {
			throw new IllegalStateException("Not logged in");
		}
		return user.getNickname();
	}

	/**
	 * ログイン済みか返す．
	 * @return
	 */
	public final boolean isLoggedIn()
	{
		if(!AppEngineUtil.isServer() || AppEngineUtil.isDevelopment()) {
			return true;
		}
		return userService.isUserLoggedIn();
	}

}
