package org.hidetake.elshard.demo.service;

import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.hidetake.elshard.demo.meta.MapReduceStateMeta;
import org.hidetake.elshard.demo.model.MapReduceState;
import org.slim3.datastore.Datastore;

import com.google.appengine.api.taskqueue.Queue;
import com.google.appengine.api.taskqueue.QueueFactory;
import com.google.appengine.api.taskqueue.TaskHandle;
import com.google.appengine.api.taskqueue.TaskOptions;
import com.google.appengine.tools.mapreduce.ConfigurationXmlUtil;

public class MapReduceService
{

	/**
	 * すべての{@link MapReduceState}を返す．
	 * @return
	 */
	public List<MapReduceState> getStateList()
	{
		return Datastore.query(MapReduceStateMeta.get()).asList();
	}

	/**
	 * {@link MapReduceState} を返す．
	 * @param jobID ジョブID
	 * @return
	 */
	public MapReduceState getMapReduceState(String jobID)
	{
		MapReduceStateMeta m = MapReduceStateMeta.get();
		return Datastore.getOrNull(m, Datastore.createKey(m, jobID));
	}

	/**
	 * ジョブを開始する．
	 * @param configuration
	 * @return
	 */
	public TaskHandle startJob(Configuration configuration)
	{
		Queue queue = QueueFactory.getDefaultQueue();
		TaskOptions task = TaskOptions.Builder
				.withMethod(TaskOptions.Method.POST)
				.url("/mapreduce/start")
				.param("configuration", ConfigurationXmlUtil.convertConfigurationToXml(configuration));
		return queue.add(task);
	}

}
