package org.hidetake.elshard.demo.service;

import java.util.Date;

import org.apache.hadoop.conf.Configuration;
import org.hidetake.elshard.demo.mapper.tweet.ParseTweet;
import org.hidetake.elshard.demo.meta.TweetMeta;
import org.slim3.memcache.Memcache;

import com.google.appengine.api.memcache.Expiration;
import com.google.appengine.api.memcache.MemcacheService.SetPolicy;

public class TweetParseJobnetService
{

	/**
	 * ジョブネットのロックを取得する．
	 * @param userName
	 * @return nullなら成功．すでにロックされている場合はロック時点の日付を返す．
	 */
	public Date lock(String userName)
	{
		if(Memcache.put(userName, new Date(), Expiration.byDeltaSeconds(600), SetPolicy.ADD_ONLY_IF_NOT_PRESENT)) {
			return null;
		}
		else {
			return (Date)Memcache.get(userName);
		}
	}

	/**
	 * ジョブネットのロックを解除する．
	 * @param userName
	 */
	public void release(String userName)
	{
		Memcache.delete(userName);
	}

	/**
	 * Mapperジョブを開始する．
	 * @param userName
	 */
	public void startMapperJob(String userName)
	{
		Configuration configuration = new Configuration();
		configuration.set(
				"mapreduce.map.class",
				"org.hidetake.elshard.demo.mapper.tweet.ParseTweet");
		configuration.set(
				"mapreduce.inputformat.class",
				"com.google.appengine.tools.mapreduce.DatastoreInputFormat");
		configuration.set(
				ParseTweet.CONFIG_USER_NAME,
				userName);
		configuration.set(
				"mapreduce.mapper.inputformat.datastoreinputformat.entitykind",
				TweetMeta.get().getKind());
		configuration.set(
				"mapreduce.appengine.donecallback.url",
				"/tweet/parse/saveCounter");
	}

}
