package org.hidetake.elshard.demo.service;

import java.util.ArrayList;
import java.util.List;

import org.hidetake.elshard.demo.meta.TweetMeta;
import org.hidetake.elshard.demo.model.Tweet;
import org.slim3.datastore.Datastore;

import twitter4j.Paging;
import twitter4j.ResponseList;
import twitter4j.Status;
import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;

import com.google.appengine.api.datastore.GeoPt;

/**
 * Tweet service.
 * 
 * @author hidetake.org
 */
public class TweetService
{

	private final TwitterFactory twitterFactory = new TwitterFactory();

	/**
	 * タイムラインを取得して保存する．
	 * @param userName
	 * @param paging
	 * @return
	 * @throws TwitterException
	 */
	public List<Tweet> fetchAndSaveTimeline(String userName, Paging paging) throws TwitterException
	{
		List<Tweet> tweets = new ArrayList<Tweet>();
		TweetMeta m = TweetMeta.get();
		
		Twitter twitter = twitterFactory.getInstance();
		ResponseList<Status> timeline = twitter.getUserTimeline(userName, paging);
		for(Status status : timeline) {
			Tweet tweet = new Tweet();
			tweet.setKey(Datastore.createKey(m, status.getId()));
			tweet.setDate(status.getCreatedAt());
			tweet.setUserName(status.getUser().getScreenName());
			tweet.setStatus(status.getText());
			tweet.setFavorite(status.isFavorited());
			tweet.setReplyTo(status.getInReplyToScreenName());
			tweet.setRetweet(status.isRetweet());
			if(status.getGeoLocation() != null) {
				tweet.setGeometry(new GeoPt(
						(float)status.getGeoLocation().getLatitude(),
						(float)status.getGeoLocation().getLongitude()));
			}
			tweets.add(tweet);
		}
		Datastore.put(tweets);
		return tweets;
	}

}
