package org.hidetake.elshard.task;

import javax.servlet.http.HttpServletRequest;

/**
 * Task context.
 * 
 * @author hidetake.org
 */
public interface Context
{

	/**
	 * Get the task chain configuration.
	 * @return
	 */
	public abstract Configuration getConfiguration();
	
	/**
	 * Get the HTTP request of this task.
	 * @return {@link HttpServletRequest} object
	 */
	public abstract HttpServletRequest getRequest();

	/**
	 * Get the identifier of this task chain.
	 * @return task chain ID
	 */
	public abstract String getTaskChainId();

	/**
	 * Get a task URL.
	 * @return 
	 */
	public abstract String getTaskUrl();

	/**
	 * Get a sequence number in the task chain.
	 * @return task chain sequence number
	 */
	public abstract int getSequenceId();

	/**
	 * Get elapsed time in millisecond.
	 * @return
	 */
	public abstract int getElapsedTimeMillis();

	/**
	 * Add a parameter for next task(s).
	 * @param key
	 * @param value
	 */
	public abstract void addParameter(String key, String value);

}
