package org.hidetake.elshard.task.impl;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.hidetake.elshard.task.Configuration;
import org.hidetake.elshard.task.Context;

/**
 * Task chain context implementation.
 * 
 * @author hidetake.org
 */
public class ContextImpl implements Context
{

	private final long startedTimeMillis;
	private final Map<String, String> params = new HashMap<String, String>();
	private final HttpServletRequest request;
	private final Configuration configuration;
	private final String taskUrl;
	private final String taskChainId;
	private final int sequenceId;
	private int taskCount = 0;

	public ContextImpl(
			HttpServletRequest request,
			Configuration configuration,
			String taskUrl,
			String taskChainId,
			int sequenceId
			)
	{
		this.startedTimeMillis = System.currentTimeMillis();
		this.request = request;
		this.configuration = configuration;
		this.taskUrl = taskUrl;
		this.taskChainId = taskChainId;
		this.sequenceId = sequenceId;
	}

	@Override
	public HttpServletRequest getRequest()
	{
		return request;
	}
	
	@Override
	public String getTaskChainId()
	{
		return taskChainId;
	}

	@Override
	public int getSequenceId()
	{
		return sequenceId;
	}

	@Override
	public int getElapsedTimeMillis()
	{
		return (int) (System.currentTimeMillis() - startedTimeMillis);
	}

	public Map<String, String> getParams()
	{
		return params;
	}

	@Override
	public String getTaskUrl()
	{
		return taskUrl;
	}
	
	@Override
	public void addParameter(String key, String value)
	{
		params.put(key, value);
	}

	/**
	 * Increment task count in this context.
	 * @return task count (after increment)
	 */
	public int incrementTaskCount()
	{
		return ++taskCount;
	}

	@Override
	public Configuration getConfiguration()
	{
		return configuration;
	}

}
