package org.hidetake.elshard.task.splitter;

import java.util.ArrayList;
import java.util.List;

import org.hidetake.elshard.task.Splitter;

/**
 * Size based splitter.
 * For example, unitSize is 1, input <code>[a, b, c]</code> will be <code>[[a], [b], [c]]</code>
 * 
 * @author hidetake.org
 */
public class SizeSplitter implements Splitter
{

	private final int unitSize;
	
	/**
	 * Constructor.
	 * @param unitSize size of a piece
	 */
	public SizeSplitter(int unitSize)
	{
		if(unitSize < 1) {
			throw new IllegalArgumentException("unitSize must be positive value");
		}
		this.unitSize = unitSize;
	}

	@Override
	public List<List<String>> split(List<String> input)
	{
		if(input == null) {
			throw new NullPointerException("input is null");
		}
		List<List<String>> result = new ArrayList<List<String>>();
		int inputSize = input.size();
		for(int startIndex = 0; startIndex < inputSize; startIndex += unitSize) {
			int endIndex = startIndex + unitSize;
			if(endIndex > inputSize) {
				endIndex = inputSize;
			}
			result.add(input.subList(startIndex, endIndex));
		}
		return result;
	}

}
