package org.hidetake.elshard.util;

import javax.servlet.http.HttpServletRequest;

/**
 * Utility class for {@link HttpServletRequest}.
 * 
 * @author hidetake.org
 */
public class RequestUtil
{

	private RequestUtil() {}

	/**
	 * Acquire value from a HTTP parameter.
	 * @param request
	 * @param key
	 * @return
	 * @throws IllegalArgumentException if key not found
	 */
	public static String getHeaderAsString(HttpServletRequest request, String key)
	{
		String value = request.getHeader(key);
		if(value == null) {
			throw new IllegalArgumentException("Header "+key+" is null");
		}
		return value;
	}

	/**
	 * Acquire value from a HTTP parameter.
	 * @param request
	 * @param key
	 * @return
	 * @throws IllegalArgumentException if key not found or invalid
	 */
	public static int getHeaderAsInt(HttpServletRequest request, String key)
	{
		String value = request.getHeader(key);
		if(value == null) {
			throw new IllegalArgumentException("Header "+key+" is null");
		}
		try {
			return Integer.parseInt(value);
		}
		catch(NumberFormatException e) {
			throw new IllegalArgumentException("Header "+key+" is invalid", e);
		}
	}

}
