/*
 * Copyright (C) 2010 hidetake.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hidetake.jsonizer;


/**
 * JavaScriptのユーティリティクラス．
 *
 */
public final class JsUtil
{

	private static final String[] ESCAPE_CHARACTERS;

	private static final char _NEXTLINE = '\n';
	private static final char _DOUBLEQUOTE = '\"';
	private static final char _BACKSLASH = '\\';

	static {
		ESCAPE_CHARACTERS = new String[_BACKSLASH+1];
		ESCAPE_CHARACTERS[_BACKSLASH] = "\\\\";
		ESCAPE_CHARACTERS[_DOUBLEQUOTE] = "\\\"";
		ESCAPE_CHARACTERS[_NEXTLINE] = "\\n";
	}

	/**
	 * 文字列をJSON出力できるようにエスケープする．
	 * @param original
	 * @return
	 */
	public static final String escape(String original)
	{
		if(original == null) {
			return null;
		}
		char[] charArray = original.toCharArray();
		StringBuilder builder = null;
		int previous = 0;
		int current = 0;
		for(char c : charArray) {
			if(c < ESCAPE_CHARACTERS.length) {
				String escaped = ESCAPE_CHARACTERS[c];
				if(escaped != null) {
					if(previous == 0) {
						builder = new StringBuilder(charArray.length + 10);
					}
					builder.append(charArray, previous, current-previous);
					builder.append(escaped);
					previous = current+1;
				}
			}
			current++;
		}
		if(previous != 0) {
			builder.append(charArray, previous, current-previous);
			return builder.toString();
		}
		else {
			return original;
		}
	}

	/**
	 * 文字をJSON出力できるようにエスケープする．
	 * @param original
	 * @return
	 */
	public static final String escape(char original)
	{
		if(original < ESCAPE_CHARACTERS.length) {
			String escaped = ESCAPE_CHARACTERS[original];
			if(escaped != null) {
				return escaped;
			}
		}
		return Character.toString(original);
	}

	private JsUtil()
	{
	}

}
