/*
 * Copyright (C) 2010 hidetake.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hidetake.jsonizer;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

/**
 * JSONize object.
 * 
 * 
 *
 */
public class Jsonizer
{

	public final class Result
	{
		private final Object target;
		
		private Result(Object target)
		{
			this.target = target;
		}
		
		public void send(HttpServletResponse response) throws IOException
		{
			response.setContentType("application/json");
			response.setCharacterEncoding("UTF-8");
			write(response.getWriter());
		}
		
		public void write(Appendable appendable) throws IOException
		{
			JsonWriter jsonWriter = new JsonWriter(appendable, map);
			jsonWriter.write(target);
		}
		
		public String toString()
		{
			try {
				StringBuilder builder = new StringBuilder();
				write(builder);
				return builder.toString();
			}
			catch (IOException e) {
				throw new RuntimeException(e);
			}
		}
	}
	
	public final class Type<M>
	{
		private final Class<M> c;
		
		public Type(Class<M> c)
		{
			this.c = c;
		}
		
		public void with(PropertyGetter<M> propertyGetter)
		{
			map.put(c, propertyGetter);
		}
	}

	private final Map<Class<?>, PropertyGetter<?>> map = new HashMap<Class<?>, PropertyGetter<?>>();

	public final <M> Type<M> type(Class<M> c)
	{
		return new Type<M>(c);
	}

	public final Result jsonize(Object object)
	{
		return new Result(object);
	}

}
